function [w,nu] = GenePEN( X, Y, A, lambda ) 
%
% [w,nu] = GenePEN( X, Y, A, lambda ) 
%
% X (n x p) the features
% Y (n x 1) the class labels {-1,+1}
% A (p x p) the (symmetric 0-1) adjacency matrix of the interaction graph
% lambda (1 x 1) the regularization constant (must be tuned by cross-validation or other method)
% w (p x 1) the weights of logistic regression
% nu (1 x 1) the intercept
%
% Requires TFOCS (http://cvxr.com/tfocs/)
%
% Copyright 2013 Nikos Vlassis
% Luxembourg Centre for Systems Biomedicine, University of Luxembourg

[n,p] = size(X);
XY = X .* repmat(Y,1,size(X,2));

% Laplacian matrix 
L = sparse(diag(A*ones(p,1))-A);

% the PEN matrix P (basicaly P=L^2 plus a constant to ensure P>=0)
I = eye(p);
J = ones(p);
P = L^2;
mu = min(0,min(vec(P))); 
P = P - mu * J + 1e-9*I;
P = P./repmat(sqrt(diag(P)),1,p)./repmat(sqrt(diag(P))',p,1);  % optional
Q = chol(P);

% TFOCS code
f1 = tfunc_scale( smooth_logreg, 1/n);
f2 = tfunc_scale( smooth_quad, lambda);

smoothF = { f1, f2 };
affineF = { linop_compose(XY, sparse([eye(p), -eye(p)])), Y; ...
            linop_compose(Q, sparse([eye(p), eye(p)])), 0 };
x0 = {zeros(2*p,1), 0};
nonsmoothF = {proj_Rplus, proj_Rn};

opts = [];
opts.tol        = 1e-16;
opts.maxits     = 3000;
opts.restart    = 100;
opts.alg    	= 'N83';

x = tfocs( smoothF, affineF, nonsmoothF, x0, opts );
w = x{1}(1:p)-x{1}(p+1:end);	% the weights
nu = x{2};                      % the intercept

